#############
# Created By: Marina Gourtovaia
# Maintainer: $Author: mg8 $
# Created On: 23 June 2010
# Last Modified: $Date: 2010-04-29 15:33:49 +0100 (Thu, 29 Apr 2010) $
# $Id: run.pm 9166 2010-04-29 14:33:49Z mg8 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/branches/prerelease-23.0/lib/npg_common/roles/run.pm $

package npg_common::config::roles::db_connect;

use Moose::Role;
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 9166 $ =~ /(\d+)/mxs; $r; };
use npg_common::config;

Readonly::Scalar our $NPG_CONF_DIR => q[.npg];

requires qw/ connection storage /;

sub _no_connection_info {
    my @info = @_;
    if (!@info) {return 1;}
    if (ref $info[0] eq q[HASH] && !$info[0]->{dsn}) {return 1;}
    return 0;
}

around 'connection' => sub {

    my $orig = shift;
    my $self = shift;
    my @info = @_;

    if ( _no_connection_info(@info) && !$self->storage) {
        my $config_file_name = ref $self;
        $config_file_name =~ s/::/-/gsmx;
        my $config = npg_common::config->new(
                                conf_dir  => $NPG_CONF_DIR,
                                conf_name => $config_file_name,
                                            );

        if (@info && ref $info[0] eq q[HASH]) {
            # that is how Catalyst passes db configuration
            $info[0]->{dsn} = $config->dsn;
            $info[0]->{user} = $config->dbuser;
            $info[0]->{password} = $config->dbpass;
	} else {
            @info = ($config->dsn, $config->dbuser, $config->dbpass);
        }
    }

    return $self->$orig(@info);
};

1;
__END__

=head1 NAME

npg_common::config::roles::db_connect

=head1 VERSION

$LastChangedRevision: 9166 $

=head1 SYNOPSIS

An example of a schema object consuming this role:
package npg_warehouse::Schema;

  use base 'DBIx::Class::Schema';
  __PACKAGE__->load_namespaces(
    result_namespace => 'Result',
  );
  BEGIN {
    use Moose;
    use MooseX::NonMoose;
    extends 'DBIx::Class::Schema';
  }
  with qw/npg_common::config::roles::db_connect/;


=head1 DESCRIPTION

A Moose role that has a modifier method for the connection method of
a class that inherits from DBIx::Class::Schema. If the connect method
of such a class is invoked without attributes, this method tries
to retrieve database dsn string, username and password from a
configuration file (see npg_common::config)

=head1 SUBROUTINES/METHODS

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Readonly

==item npg_common::config

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: mg8 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Marina Gourtovaia

This file is part of NPG.

NPG is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut
