#############
# $Id: mq.pm 13725 2011-07-21 15:26:34Z ajb $
# Created By: ajb
# Mast Maintained By: $Author: ajb $
# Created On: 2009-05-21
# Last Changed On: $Date: 2011-07-21 16:26:34 +0100 (Thu, 21 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-37.0/lib/npg_common/mq.pm $

package npg_common::mq;
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use srpipe::util;
use Class::Std;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 13725 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $DEFAULT_CLIENT_ID => q[npg];
Readonly::Scalar our $DEFAULT_QUEUE     => q[npg];
Readonly::Scalar our $DEFAULT_PORT      => q[61613];
Readonly::Scalar our $DEFAULT_HOST      => q[localhost];
Readonly::Scalar our $DEFAULT_ACK       => q[client];
Readonly::Scalar our $DEFAULT_DURABLE   => q[true];
Readonly::Scalar our $DEFAULT_LOGIN     => q[guest];
Readonly::Scalar our $DEFAULT_PASSCODE  => q[guest];
Readonly::Scalar our $DEFAULT_RETRIES   => 10;
Readonly::Scalar our $DEFAULT_DELIVERY_MODE => 2;
{
  my %util_of                           :ATTR( :get<util>,            :set<util>            );
  my %connection_of                     :ATTR( :get<connection>,      :set<connection>      );
  my %client_id_of                      :ATTR( :get<client_id>,       :set<client_id>       );
  my %just_sent_of                      :ATTR( :get<just_sent>,       :set<just_sent>       );
  my %host_for_force_disconnect_of      :ATTR( :get<force_host>,      :set<force_host>      );
  my %port_for_force_disconnect_of      :ATTR( :get<force_port>,      :set<force_port>      );
  my %client_id_for_force_disconnect_of :ATTR( :get<force_client_id>, :set<force_client_id> );

  sub util {
    my ($self, $util) = @_;

    if ($util) {
      $self->set_util($util);
    }

    if (!$self->get_util()) {
      $util = srpipe::util->new();
      $self->set_util($util);
    }

    return $self->get_util();
  }

  sub _default_client_id {
    return $DEFAULT_CLIENT_ID;
  }

  sub _default_login {
    return $DEFAULT_LOGIN;
  }

  sub _default_passcode {
    return $DEFAULT_PASSCODE;
  }

  sub _default_ack {
    return $DEFAULT_ACK;
  }

  sub _default_durable {
    return $DEFAULT_DURABLE;
  }

  sub _default_queue {
    return $DEFAULT_QUEUE;
  }

  sub _default_host {
    return $DEFAULT_HOST;
  }

  sub _default_port {
    return $DEFAULT_PORT;
  }

  sub _default_retries {
    return $DEFAULT_RETRIES;
  }

  sub _default_delivery_mode {
    return $DEFAULT_DELIVERY_MODE;
  }

  sub delete_connection {
    my ($self, $arg_refs) = @_;
    $arg_refs ||= {};
    $self->util->log(q{**** 'in delete_connection' ****});
    if ($self->get_connection() && $self->get_connection()->socket->connected() && !$self->get_just_sent()) {
      $self->util->log(q{**** 'going to disconnect' ****});
      my $retries = $arg_refs->{retries} || $self->_default_retries();
      my $retry_count = 0;
      while ($retry_count < $retries) {
        $retry_count++;
        eval {
          $self->util->log(qq{**** 'attempt to disconnect $retry_count' ****});
          $self->util->log(q{**** '} . $self->get_connection() . q{' ****});
          my $response_to_disconnect = $self->get_connection()->disconnect();

          my $error = $self->_check_frame_for_error($response_to_disconnect, $arg_refs);
          if ($error) {
            croak qq{Error in disconnect frame:\n\t$error};
          }

          $self->util->log(q{Disconnected }.$self->get_connection());
          $retry_count = $retries + 1;
          1;
        } or do {
          $self->util->log(qq{\tProblem attempting to disconnect:\n\t$EVAL_ERROR\n\nRetrying...});
        };
      }
      if ($retry_count == $retries) {
        $self->util->log(qq{\tUnable to disconnect cleanly, just killing connection object after $retries times\n\n});
      }
    }
    $self->util->log(q{Deleting connection object});
    $self->set_connection(undef);
    sleep 1;
    return 1;
  }

  sub _process_message_with_supplied_code {
    my ($self, $message, $code) = @_;
    if (!$code || ref$code ne q{CODE}) {
      return;
    }
    my $value;
    eval {
      $value = $code->($message);
      1;
    } or do {
      croak qq{Problem running code with message $message: $EVAL_ERROR};
    };
    return $value;
  }

  sub _check_frame_for_error {
    my ($self, $frame, $arg_refs) = @_;
    if (!ref$frame) {
      return;
    }
    my @info = split /\n/xms, $frame->as_string();
    if ($info[0] ne q{ERROR}) {
      return;
    }

    my ($client_id, $host, $port) = $info[1] =~ m{Client:[ ](.*?)[ ]already[ ]connected[ ]from[ ]/(.*?):(\d+)}xms;
    $self->set_force_host($host);
    $self->set_force_port($port);
    $self->set_force_client_id($client_id);
    if ($client_id) {
      $self->_force_disconnect($arg_refs);
    }
    return $info[1] . qq{\n\n\tmore can be found in the mq logs\n};
  }

  sub _force_disconnect {
    my ($self, $arg_refs) = @_;
    $self->util->log(q{**** Placeholder for _force_disconnect - apply this in your subclass ****});
    return 1;
  }

  sub DESTROY {
    my ($self) = @_;
    if ($self->get_connection()) {
      my $util = $self->util();
      $util->log(q{Closing down connection: }. $self->get_connection());
      $self->delete_connection();
    }

    delete $util_of{ident $self};
    delete $client_id_of{ident $self};
    delete $connection_of{ident $self};
    delete $just_sent_of{ident $self};
    delete $host_for_force_disconnect_of{ident $self};
    delete $port_for_force_disconnect_of{ident $self};
    delete $client_id_for_force_disconnect_of{ident $self};
    return;
  }

}

1;
__END__

=head1 NAME

  npg_common::mq

=head1 VERSION

$Revision: 13725 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 util - accessor for an srpipe::util object by default, although an alternative could be provided

  my $oUtil = $oDerivedClass->util();
  $oDerivedClass->util($oUtil);

=head2 delete_connection - method which will clear out any connection which may exist

  $oDerivedClass->delete_connection();

=head2 DESTROY - on cleanup of object, disconnects from mq

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item Carp

=item English -no_match_vars

=item Class::Std

=item Readonly

=item base

=item srpipe::util

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
