#############
# $Id: stomp.pm 13725 2011-07-21 15:26:34Z ajb $
# Created By: ajb
# Mast Maintained By: $Author: ajb $
# Created On: 2009-05-21
# Last Changed On: $Date: 2011-07-21 16:26:34 +0100 (Thu, 21 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-37.0/lib/npg_common/mq/stomp.pm $

package npg_common::mq::stomp;
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};

use Class::Std;
use Net::Stomp;
use Net::Stomp::Frame;
use Net::Stomp::Receipt;
use Net::Stomp::Receipt::Connected;
use base qw{npg_common::mq};

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13725 $ =~ /(\d+)/mxs; $r; };

{
  sub connection {
    my ($self, $connection_params) = @_;

    if ($connection_params && $self->get_connection()) {
      $self->util->log(q{Disconnecting previous connection: } . $self->get_connection());
      $self->delete_connection($connection_params);
    }

    if ($self->get_connection()) {
      return $self->get_connection();
    }

    $connection_params ||= {};

    my $hostname  = $connection_params->{hostname}  || $self->_default_host();
    my $port      = $connection_params->{port}      || $self->_default_port();
    my $client_id = $connection_params->{client_id} || $self->_default_client_id();
    my $login     = $connection_params->{login}     || $self->_default_login();
    my $passcode  = $connection_params->{passcode}  || $self->_default_passcode();

    my $stomp;
    $self->util->log("Connecting to $hostname  $port");
    eval {
      $stomp = Net::Stomp::Receipt::Connected->new( { hostname => $hostname, port => $port, q{client-id} => $client_id, PERSISTENT => 1, });
      my $frame = $stomp->connect({ login => $login, passcode => $passcode, q{client-id} => $client_id });
      my $error = $self->_check_frame_for_error($frame, $connection_params);
      if ($error) {
        croak qq{Error in connection frame:\n\t$error};
      }
      1;
    } or do {
      $self->util->log("Cannot connect to a message queue server: $EVAL_ERROR\n");
      $self->delete_connection($connection_params);
      croak "Cannot connect to a message queue server: $EVAL_ERROR\n";
    };
    $self->util->log(q{Connection made: } . $stomp);

    $self->set_connection($stomp);
    return $self->get_connection();
  }

  sub _force_disconnect {
    my ($self, $arg_refs) = @_;
    eval {
      $self->util->log(q{**** Attempting to force disconnect }.$self->get_force_client_id().q{ ****});
      my $admin = Net::Stomp->new({
          hostname => ($arg_refs->{hostname} || $self->_default_host()),
          port => ($arg_refs->{port} || $self->_default_port()),
          client_id => q{admin},
        });
      $admin->connect();
      my $disconnect_frame = Net::Stomp::Frame->new({
          headers => {
            client_id => $self->get_force_client_id(),
            hostname => $self->get_force_host(),
            port => $self->get_force_port(),
          },
          command => q{DISCONNECT},
        });
      $admin->send_frame($disconnect_frame);
      $admin->disconnect();
      1;
    } or do {
      croak qq{Unable to force disconnect:\n\t}.$self->get_force_client_id().qq{\n\t}.$self->get_force_host().qq{\n\t}.$self->get_force_port().qq{\n\n$EVAL_ERROR};
    };
    return 1;
  }

}
1;
__END__

=head1 NAME

  npg_common::mq::stomp

=head1 VERSION

$LastChangedRevision: 13725 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 connection - obtains and/or returns a Net::Stomp object with a connection made with optional parameters provided, or defaults used

  my $oConnection = $oDerivedClass->connection({
    hostname => $sHostName,
    port => $sPort,
    client-id => $sClientId,
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item Carp

=item English -no_match_vars

=item Class::Std

=item Readonly

=item base

=item npg_common::mq

=item Net::Stomp

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
