#############
# $Id: topic.pm 13725 2011-07-21 15:26:34Z ajb $
# Created By: ajb
# Mast Maintained By: $Author: ajb $
# Created On: 2009-05-21
# Last Changed On: $Date: 2011-07-21 16:26:34 +0100 (Thu, 21 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-37.0/lib/npg_common/mq/stomp/topic.pm $

package npg_common::mq::stomp::topic;
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};

use Class::Std;
use base qw{npg_common::mq::stomp};

Readonly::Scalar our $DEFAULT_TOPIC       => q[/topic/npg];
Readonly::Scalar our $DEFAULT_EXCHANGE    => q[amq.topic];
Readonly::Scalar our $DEFAULT_PERSISTENT  => q[true];

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13725 $ =~ /(\d+)/mxs; $r; };

{

  my %topic_of            :ATTR( :get<topic>,           :set<topic>           );

  sub send { ## no critic (Subroutines::ProhibitBuiltinHomonyms)
    my ($self, $arg_refs) = @_;
    my $topic = $arg_refs->{topic} || $arg_refs->{destination} || $DEFAULT_TOPIC;
    my $body = $arg_refs->{message} || $arg_refs->{body};
    my $exchange = $arg_refs->{exchange} || $DEFAULT_EXCHANGE;
    my $persistent = $arg_refs->{persistent} || $DEFAULT_PERSISTENT;
    my $delivery_mode = $arg_refs->{delivery_mode} || $self->_default_delivery_mode();
#warn qq{topic key: } . $arg_refs->{topic};
#warn qq{destination key: } . $arg_refs->{destination};
    if (!$body) {
      $self->delete_connection($arg_refs);
      croak q{No message text};
    }

    $self->util->log(qq{Attempting to send message:\ntopic:$topic\nexchange:$exchange\npersistent:$persistent\nbody:\n$body\n});

    my $response;
    eval {
      my $connection = $arg_refs->{connection} || $self->connection($arg_refs);
      $connection->send_safe({
        destination => $topic,
        exchange => $exchange,
        persistent => $persistent,
        body => $body,
      }) or croak qq{Error trying to send_safe:\n\t$EVAL_ERROR};

      my $error = $self->_check_frame_for_error($response);
      if ($error) {
        croak qq{Error in sending frame:\n\t$error};
      }
      1;
    } or do {
      $self->delete_connection($arg_refs);
      croak qq{Unable to send message:\n\t$EVAL_ERROR};
    };

    $self->set_just_sent($response);
    $self->util->log(q{message sent});
    $self->delete_connection();
    return 1;
  }

  sub subscribe {
    my ($self, $arg_refs) = @_;
    $arg_refs ||= {};

    my $topic     = $arg_refs->{topic}     || $arg_refs->{destination}     || $DEFAULT_TOPIC;
    my $exchange  = $arg_refs->{exchange}  || $DEFAULT_EXCHANGE;
    my $client_id = $arg_refs->{client_id} || $self->_default_client_id();
    my $durable   = $arg_refs->{durable}   || $self->_default_durable();
    my $ack       = $arg_refs->{ack}       || $self->_default_ack();

    $self->set_topic($topic);
    $self->set_client_id($client_id);

    my $connection;
    my $retries = $arg_refs->{retries} || $self->_default_retries();
    my $retry_count = 0;
    while ($retry_count < $retries) {
      $retry_count++;
      $self->util->log(qq{Attempt to subscribe to $topic: $retry_count out of $retries});
      eval {
        $self->util->log(q{**** 'deleting connection' ****});
        $self->delete_connection();
        $self->util->log(q{**** 'making connection' ****});
        $connection = $self->connection($arg_refs);
        $self->util->log(q{**** 'creating subscription' ****});
        my $response = $connection->subscribe({
          destination => $topic,
          durable     => $durable,
          ack         => $ack,
          q{auto-delete} => q{false},
          q{activemq.subscriptionName} => $topic.q{_subscriber},
          });
        my $error = $self->_check_frame_for_error($response, $arg_refs);
        if ($error) {
          croak qq{Error in subscription frame:\n\t$error};
        }

      } or do {
        $self->util->log(qq{\tUnable to subscribe to topic $topic as $client_id:\n\t$EVAL_ERROR\n\nRetrying...});
      };
      $retry_count = $retries + 1;
    }
    if ($retry_count == $retries) {
      $self->delete_connection($arg_refs);
      croak qq{\tUnable to subscribe to topic $topic as $client_id:\n\t$EVAL_ERROR\n\n};
    }
    $self->util->log("Subscribed to $topic as $client_id");
    return $connection;
  }

  sub _unsubscribe {
    my ($self, $arg_refs) = @_;
    if (!$self->get_connection() || !$self->get_client_id()) {
      return 1;
    }
    eval {
      my $response = $self->get_connection()->unsubscribe({
        destination => $self->get_topic(),
        q(activemq.subscriptionName) => $self->get_client_id(),
      });
      my $error = $self->_check_frame_for_error($response, $arg_refs);
      if ($error) {
        croak qq{Error in unsubscription frame:\n\t$error};
      }

      1;
    } or do {
      $self->util->log(qq{Problem unsubscribing: $EVAL_ERROR});
    };
    return 1;
  }


  sub get_message {
    my ($self, $arg_refs) = @_;
    my $util = $self->util;

    $self->set_just_sent(undef);

    if ($arg_refs->{code}) {
      $arg_refs->{ack} = q{client}; # force client acknowledgement if code provided to be acted upon
    }

    my $topic     = $arg_refs->{topic}        || $arg_refs->{destination}     || $DEFAULT_TOPIC;
    my $client_id = $arg_refs->{client_id}    || $self->_default_client_id();
    my $ack       = $arg_refs->{ack}          || $self->_default_ack();
    my $retries   = $arg_refs->{retries}      || $self->_default_retries();

    my $message;
    my $process_value;
    eval {
      my $connection = $self->subscribe($arg_refs);

      $util->log(qq{Retrieving messages for topic $topic using $connection});

      my $retry_count = 0;
      while ($retry_count < $retries) {
        if ($retry_count != 0) { $connection = $self->subscribe($arg_refs); }
        $retry_count++;
        eval {
          while ($connection->can_read({ timeout => '0.1' })) {
            my $frame = $connection->receive_frame;

            $message = $frame->body();

            ###############
            # routine to operate on CODE before acknowledging receipt
            $process_value = $self->_process_message_with_supplied_code($message, $arg_refs->{code});

            if ($ack eq q{client}) {
              $connection->ack( { frame => $frame });
            }
            last if $frame;
          }
          $retry_count = $retries + 1;
          1;
        } or do {
          my $log = qq{Unable to obtain a message from topic $topic as $client_id:\n\t$EVAL_ERROR};
          my $count = $retry_count + 1;
          if ($retry_count < $retries) {
            $log .= qq{\n\nRetrying $count of $retries};
          }
          $message = undef; # ensure that we don't report the wrong thing below in the logs, in case the error is croaking in supplied code block
          $self->util->log($log);
        };
      }
      $self->delete_connection($arg_refs);
    } or do {
      $util->log(qq{Problem obtaining a message: $EVAL_ERROR});
    };

    if (!$message) {
      $util->log(qq{no messages for topic $topic at this time});
      return;
    }
    if ($process_value && $process_value != 1) {
      $util->log(q{Returning processed message value});
      return $process_value;
    }
    $util->log(qq{Returning message from topic $topic});
    return $message;
  }

  sub DESTROY {
    my ($self) = @_;
    if ($self->get_connection()) {
      my $util = $self->util();
      $util->log(q{Closing down connection: }. $self->get_connection());
      $self->delete_connection();
    }

    delete $topic_of{ident $self};
    return;
  }

}

1;
__END__

=head1 NAME

  npg_common::mq::stomp::topic

=head1 VERSION

$LastChangedRevision: 13725 $

=head1 SYNOPSIS

  my $oTopic = npg_common::mq::stomp::topic->new();

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 send - method to send a message to a topic. takes a hashref of keyvalue pairs to create connection and use as headers

  eval {
    $oTopic->send({
      destination => q{uri for topic}, ## defaults to q{/topic/npg}
      topic => q{uri for topic}, ## alternative for destination
      exchange => q{exchange}, ## defaults to amq.topic
      q{activemq.subscriptionName} => q{client_id}, ## defaults to q{npg}, this needs to be unique for each subscriber to the topic
      client_id => q{client_id}, ## alternative for q{activemq.subscriptionName}
      persistent => q{true/false}, ## defaults to q{true}
      hostname => q{hostname}, ## defaults to q{localhost}
      port => q{port}, ## defaults to 61613
      login => q{login}, ## defaults to q{guest}
      passcode => q{passcode}, ## defaults to q{guest}
      ack => q{} || q{client}, ## defaults to q{client}
      body => q{message body},
      message => q{message body}, ## alternative to body
    });
  } or do {
    croak $EVAL_ERROR;
  };

  will croak if no message body provided

=head2 subscribe - connects and subscribes to a topic, returning the connection

  my $connection;
  eval {
    $connection = $oTopic->subscribe({
      destination => q{uri for topic}, ## defaults to q{/topic/npg}
      topic => q{uri for topic}, ## alternative for destination
      exchange => q{exchange}, ## defaults to amq.topic
      q{activemq.subscriptionName} => q{client_id}, ## defaults to q{npg}
      client_id => q{client_id}, ## alternative for q{activemq.subscriptionName}
      durable => q{true/false}, ## defaults to q{true}
      hostname => q{hostname}, ## defaults to q{localhost}
      port => q{port}, ## defaults to 61613
      login => q{login}, ## defaults to q{guest}
      passcode => q{passcode}, ## defaults to q{guest}
    });
    1;
  } or do {
    croak $EVAL_ERROR;
  };

=head2 get_message - retrieves a message body from a topic, or returns undef if nothing on topic to be read/obtained (automatically handles subscription)

  my $message;
  eval {
    $message = $oTopic->get_message({
      destination => q{uri for topic}, ## defaults to q{/topic/npg}
      topic => q{uri for topic}, ## alternative for destination
      exchange => q{exchange}, ## defaults to amq.topic
      q{activemq.subscriptionName} => q{client_id}, ## defaults to q{npg}
      client_id => q{client_id}, ## alternative for q{activemq.subscriptionName}
      durable => q{true/false}, ## defaults to q{true}
      hostname => q{hostname}, ## defaults to q{localhost}
      port => q{port}, ## defaults to 61613
      login => q{login}, ## defaults to q{guest}
      passcode => q{passcode}, ## defaults to q{guest}
      code => sub { my $message = shift; < do stuff >; return 1; }, ## optional code block to be executed with message, the value of which will be returned, instead of the message, if it is true and not 1
    });
    1;
  } or do {
    croak $EVAL_ERROR;
  };

=head2 DESTROY - on cleanup of object, ensures unsubscription (if applicable) and disconnection

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item Carp

=item English -no_match_vars

=item Class::Std

=item Readonly

=item base

=item npg_common::mq::stomp

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
