#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2009-04-17
# Last Modified: $Date: 2011-07-21 16:26:34 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 10-Alignment.t 13725 2011-07-21 15:26:34Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-37.0/t/10-Alignment.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 8;
use npg_common::Alignment;

use_ok('npg_common::Alignment');

{
  my $bwa = npg_common::Alignment->new(
               {
                 bwa_cmd => q{t/bin/bwa},
                 samtools_cmd => q{t/bin/samtools},
               });
               
  isa_ok($bwa, 'npg_common::Alignment', 'object test');
  
  my $reference_bwa = '/nfs/repository/d0031/references/Human/NCBI36/all/bwa/Homo_sapiens.NCBI36.48.dna.all.fa';
  my $query_fastq = 't/data/fastq_split/2605_1_1.fastq';
  my $sam = '/tmp/2605_1_1.sam';

  eval{
    $bwa->bwa_align_se({fastq=>$query_fastq, bam_out=>$sam, ref_root=>$reference_bwa});
    1;
  };
  is($EVAL_ERROR, q{}, "no croak for bwa alignment single");

  eval{
    $bwa->bwa_align_pe({fastq1=>$query_fastq, fastq2=>$query_fastq, sam_out=>$sam, fork_align=>1, ref_root=>$reference_bwa});
    1;
  };
  is($EVAL_ERROR, q{}, "no croak for bwa alignment paired ");

  my $reference_fai = '/nfs/repository/d0031/references/Human/NCBI36/all/samtools/Homo_sapiens.NCBI36.48.dna.all.fa.fai';
  my $bam = '/tmp/2605_1_1.bam';
  eval{
    $bwa->sam2bam($sam, $bam, $reference_fai);
    1;
  };
  is($EVAL_ERROR, q{}, "no croak when converting sam to bam");
  
  my $sorted_bam_filename = '/tmp/2605_1_1.sorted';
  my $sored_bam_fullname = '/tmp/2605_1_1.sorted.bam';
  my $pileup_filename = '/tmp/2605_1_1.pileup';
  `touch $bam`;
  eval{
    $bwa->sort_bam($bam, $sorted_bam_filename);
    1;
  };
  is($EVAL_ERROR, q{}, "no croak when sorting bam");
 
  my $reference_fa = 't/data/fastq_split/Homo_sapiens.NCBI36.48.dna.all.fa'; 
  eval{
    $bwa->parsing_pileup_bam($sored_bam_fullname, $reference_fa, $pileup_filename);
    1;
  };
  is($EVAL_ERROR, q{}, "no croak when parsing pileup stream");

  
  eval{
    $bwa->alignment_coverage_from_sam($sam, $reference_fa);
    1;
  };
  is($EVAL_ERROR, q{}, "no croak when check alignment from sam");
   
  system qq(rm -rf $bam $sam $sam.sai $sored_bam_fullname $pileup_filename);
}
