#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2009-04-17
# Last Modified: $Date: 2011-07-21 16:26:34 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 10-sequence-BAM_Generation.t 13725 2011-07-21 15:26:34Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-37.0/t/10-sequence-BAM_Generation.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 27;
use Test::Exception;
use Cwd;
use File::Spec::Functions qw(catfile);
use npg_common::sequence::reference::roles::find;
use lib q{t};
use t::util;

use_ok('npg_common::sequence::BAM_Generation');

my $util = t::util->new();
my $cache = q[t/data/sequence];
my $cache_var_name = q[NPG_WEBSERVICE_CACHE_DIR];
local $ENV{$cache_var_name} = $cache;
my $repos = catfile(cwd, q[t/data/sequence/references]);

{
  my $bam = npg_common::sequence::BAM_Generation->new(
               {
                 id_run    => 4394,
                 position  => 1,
                 fastq_files => ['t/data/sequence/SecondCall_paired/4394_1_1.fastq', 't/data/sequence/SecondCall_paired/4394_1_2.fastq'],
                 bam_filename   => '/tmp/4394_1',
                 second_call_qseq_dir  => 't/data/sequence/SecondCall_paired',
                 original_quality_qseq_dir  => 't/data/sequence/SecondCall_paired',
                 read_number_1         => 1,
                 read_number_2         => 2,
               });
               
  isa_ok($bam, 'npg_common::sequence::BAM_Generation', 'object test');

  my $bwa_cmd = 't/bin/aligners/bwa/current';
  $bam->aligner_cmd($bwa_cmd);
  is( $bam->aligner_cmd(), getcwd().q{/t/bin/aligners/bwa/bwa-0.5.8c}, 'convert bwa path to absolute path');
  
  my $samtools_cmd = 't/bin/aligners/samtools/current';
  $bam->samtools_cmd($samtools_cmd);
  is( $bam->samtools_cmd(), getcwd().q{/t/bin/aligners/samtools/samtools-0.1.9}, 'convert bwa path to absolute path');

  my $picard_cmd = 't/bin/aligners/picard/current VALIDATION_STRINGENCY=SILENT INPUT=/dev/stdin';
  $bam->picard_converter_cmd($picard_cmd);
  is( $bam->picard_converter_cmd(), getcwd().q{/t/bin/aligners/picard/picard-tools-1.31 VALIDATION_STRINGENCY=SILENT INPUT=/dev/stdin}, 'convert picard path to absolute path');

  my $ref_info = undef;
  $bam->reference_info($ref_info);

  ok( !$bam->reference(), 'no reference return for run 4394 lane 1');
  is( $bam->aligner_options(), '-q 15 -t 2', 'default aligner options return for run 4394 lane 1');
  is($bam->no_alignment, 0, 'false for no alignment');
 
  my $cmd_options = q{--lane 1 --qseq_dir t/data/sequence/SecondCall_paired --read_number_1 1 --read_number_2 2};
 
  is( $bam->_second_call_options(), $cmd_options, 'correct second call command options') ;
  is( $bam->_original_quality_options(), $cmd_options, 'correct original quality command options');
}

{
  my $bam = npg_common::sequence::BAM_Generation->new({
                 id_run          => 5175,
                 position        => 1,
                 tag_index       => 1,
                 fastq_files     => ['t/data/sequence/SecondCall_paired/4394_1_1.fastq'],
                 bam_filename    => '/tmp/5175_1#1',
               });

  isa_ok( $bam, 'npg_common::sequence::BAM_Generation', 'object test' );
  
  my $ref_info = npg_common::sequence::reference->new(id_run => 5175, position  => 1,tag_index => 1, ref_repository  => $repos)
                                                ->ref_info();
  $bam->reference_info($ref_info);

  isa_ok( $bam->reference_info(), 'npg_common::sequence::reference::info', 'reference_info object' );
  is($bam->reference(),  $repos.q{/Bordetella_bronchiseptica/RB50/all/bwa/ref.fa}, 'correct reference path returned for run 5175 lane 1 tag 1');
  is($bam->aligner_options(), q{ -t 2} , 'no special bwa options returned for run 5175 lane 1 tag 1');
  is($bam->no_alignment, 0, 'false for no alignment');
}

{
  my $bam = npg_common::sequence::BAM_Generation->new({
                 id_run          => 5175,
                 position        => 1,
                 tag_index       => 1,
                 no_alignment    => 1,
                 fastq_files     => ['t/data/sequence/SecondCall_paired/4394_1_1.fastq'],
                 bam_filename    => '/tmp/5175_1#1',
               });
  is($bam->no_alignment, 1, 'no alignment option as set');
}

{
  local $ENV{$cache_var_name} = q[t/data/sequence1];
  my $bam = npg_common::sequence::BAM_Generation->new(
               {
                 id_run    => 4394,
                 position  => 1,
                 fastq_files => ['t/data/sequence/SecondCall_paired/4394_1_1.fastq', 't/data/sequence/SecondCall_paired/4394_1_2.fastq'],
                 bam_filename   => '/tmp/4394_1',
                 second_call_qseq_dir  => 't/data/sequence/SecondCall_paired',
                 read_number_1         => 1,
                 read_number_2         => 2,
               });
               
  isa_ok($bam, 'npg_common::sequence::BAM_Generation', 'object test');
  is($bam->min_read_length_to_align, 31,' default min_read_length_to_align');
  is($bam->no_alignment, 0, 'false for no alignment');

  $bam = npg_common::sequence::BAM_Generation->new(
               {
                 id_run    => 4394,
                 position  => 1,
                 fastq_files => ['t/data/sequence/SecondCall_paired/4394_1_1.fastq', 't/data/sequence/SecondCall_paired/4394_1_2.fastq'],
                 bam_filename   => '/tmp/4394_1',
                 second_call_qseq_dir  => 't/data/sequence/SecondCall_paired',
                 read_number_1         => 1,
                 read_number_2         => 2,
                 min_read_length_to_align => 56,
               });
  is($bam->min_read_length_to_align, 56,' min_read_length_to_align 56 as set');
  is($bam->no_alignment, 1, 'true for no alignment, reads are too short');

  $bam = npg_common::sequence::BAM_Generation->new(
               {
                 id_run    => 4394,
                 position  => 1,
                 fastq_files => ['t/data/sequence/SecondCall_paired/4394_1_1.fastq'],
                 bam_filename   => '/tmp/4394_1',
                 second_call_qseq_dir  => 't/data/sequence/SecondCall_paired',
                 read_number_1         => 1,
                 read_number_2         => 2,
                 min_read_length_to_align => 56,
               });
  is($bam->no_alignment, 1, 'true for no alignment, single read is too short');


  my $dir = $util->temp_directory();
  `touch $dir/file1`;
  $bam = npg_common::sequence::BAM_Generation->new(
               {
                 id_run    => 4394,
                 position  => 1,
                 fastq_files => ["$dir/file1"],
                 bam_filename   => '/tmp/4394_1',
                 second_call_qseq_dir  => 't/data/sequence/SecondCall_paired',
                 read_number_1         => 1,
                 read_number_2         => 2,
               });
  is($bam->no_alignment, 1, 'true for no alignment, single read is empty');

  $bam = npg_common::sequence::BAM_Generation->new(
               {
                 id_run    => 4394,
                 position  => 1,
                 fastq_files => ['t/data/sequence/SecondCall_paired/4394_1_1.fastq',"$dir/file1"],
                 bam_filename   => '/tmp/4394_1',
                 second_call_qseq_dir  => 't/data/sequence/SecondCall_paired',
                 read_number_1         => 1,
                 read_number_2         => 2,
               });
  is($bam->no_alignment, 1, 'true for no alignment, second read is empty');
}

{
  local $ENV{$cache_var_name} = q[t/data/sequence2];
  my $bam = npg_common::sequence::BAM_Generation->new(
               {
                 id_run    => 4394,
                 position  => 1,
                 fastq_files => ['t/data/sequence/SecondCall_paired/4394_1_1.fastq', 't/data/sequence/SecondCall_paired/4394_1_2.fastq'],
                 bam_filename   => '/tmp/4394_1',
                 second_call_qseq_dir  => 't/data/sequence/SecondCall_paired',
                 read_number_1         => 1,
                 read_number_2         => 2,
               });
               
  isa_ok($bam, 'npg_common::sequence::BAM_Generation', 'object test');
  is($bam->no_alignment, 1, 'true for no alignment');

  $bam = npg_common::sequence::BAM_Generation->new(
               {
                 id_run    => 4394,
                 position  => 1,
                 fastq_files => ['t/data/sequence/SecondCall_paired/4394_1_1.fastq', 't/data/sequence/SecondCall_paired/4394_1_2.fastq'],
                 bam_filename   => '/tmp/4394_1',
                 second_call_qseq_dir  => 't/data/sequence/SecondCall_paired',
                 read_number_1         => 1,
                 read_number_2         => 2,
                 no_alignment          => 0,
               });
  is($bam->no_alignment, 0, 'false for no alignment as set');
}

1;
