#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: ajb $
# Created:       June 2010
# Last Modified: $Date: 2011-07-21 16:26:34 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 10-sequence-reference.t 13725 2011-07-21 15:26:34Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-37.0/t/10-sequence-reference.t $
#

package reference;

use strict;
use warnings;
use Test::More tests => 84;
use Test::Exception;
use Test::Warn;
use Test::Deep;
use File::Spec::Functions qw(catfile);
use Cwd qw(cwd);
use Moose::Meta::Class;

use st::api::batch;
use st::api::sample;

my $repos = catfile(cwd, q[t/data/sequence/references]);
my $cache = q[t/data/sequence];
my $cache_var_name = q[NPG_WEBSERVICE_CACHE_DIR];

local $ENV{$cache_var_name} = $cache;

use_ok('npg_common::sequence::reference::info');
use_ok('npg_common::sequence::reference::roles::list');
use_ok('npg_common::sequence::reference::roles::find');
use_ok('npg_common::sequence::reference');

{
  my $ruser = npg_common::sequence::reference->new(id_run => 1937, position => 1, ref_repository => $repos);
  isa_ok($ruser, 'npg_common::sequence::reference');
}


{
  lives_ok {npg_common::sequence::reference->new(id_run => 1937, position => 1, tag_index => 4)} 'can take tag index in the constructor';
}


{
  my $ruser = npg_common::sequence::reference->new(position => 8, id_run => 1937, ref_repository => $repos, aligner => q[eland]);
  is( pop @{$ruser->refs}, catfile($repos, 'PhiX/default/all/eland/phix-illumina.fa'), 'phix reference for eland');
  is ($ruser->messages->count, 0, 'no messages');

  $ruser = npg_common::sequence::reference->new(position => 8, id_run => 1937, ref_repository => $repos, aligner => q[bowtie]);
  is( pop @{$ruser->refs}, catfile($repos, 'PhiX/default/all/bowtie/phix-illumina.fa'), 'phix reference for bowtie');
  is ($ruser->messages->count, 0, 'no messages');

  $ruser = npg_common::sequence::reference->new(position => 8, id_run => 1937, ref_repository => $repos, aligner => q[bwa]);
  is( pop @{$ruser->refs}, catfile($repos, 'PhiX/default/all/bwa/phix-illumina.fa'), 'phix reference for bwa explicitly');
  is ($ruser->messages->count, 0, 'no messages');

  $ruser = npg_common::sequence::reference->new(position => 8, id_run => 1937, 
                                                ref_repository => $repos, aligner => q[bwa],
                                                species => q[Human],
                                               );
  is( $ruser->refs->[0], catfile($repos, q[Human/NCBI36/all/bwa/someref.fa]), 'human reference overwrite when species is set');

  $ruser = npg_common::sequence::reference->new(position => 8, id_run => 1937, 
                                                ref_repository => $repos, aligner => q[bwa],
                                                reference_genome => q[Human (NCBI36)],
                                               );
  is( $ruser->refs->[0], catfile($repos, q[Human/NCBI36/all/bwa/someref.fa]), 'human reference overwrite when reference_genome is set');
}


{
  my $ruser = npg_common::sequence::reference->new(id_run => 4140, position => 1, ref_repository => $repos,
                          search_type =>  $npg_common::sequence::reference::roles::find::SEARCH_FUZZY);
  is (pop @{$ruser->refs()}, catfile($repos, 'Human/NCBI36/all/bwa/someref.fa'), 'human ref from the project name');
  is ($ruser->messages->pop, 'Organism derived from study name', 'organism derived from study name message');
  is ($ruser->messages->pop, 'no reference for taxon id 23003', 'searched taxon_id message');
}


{
  my $ruser = npg_common::sequence::reference->new(position => 8, id_run => 1937, ref_repository => $repos,
                search_type =>  $npg_common::sequence::reference::roles::find::SEARCH_FUZZY);
  is( pop @{$ruser->refs}, catfile(cwd, 't/data/sequence/references/PhiX/default/all/bwa/phix-illumina.fa'), 'phix reference for lane 8');
  is( $ruser->messages->count, 0, 'no messages');


  $ruser = npg_common::sequence::reference->new(position => 4, id_run => 1937, ref_repository => $repos,
                search_type =>  $npg_common::sequence::reference::roles::find::SEARCH_FUZZY);
  #$ruser->position(4);
  is( pop @{$ruser->refs}, catfile($repos, 'PhiX/default/all/bwa/phix-illumina.fa'), 'phix reference for lane 4');
  is( $ruser->messages->count, 0, 'no messages' );

  # sample id 1986, organism=Human, taxon_id=9606, Common_Name = 'Homo sapiens';
  $ruser = npg_common::sequence::reference->new(position => 1, id_run => 1937, ref_repository => $repos,
                search_type =>  $npg_common::sequence::reference::roles::find::SEARCH_FUZZY);
  #$ruser->position(1);
  is( pop @{$ruser->refs}, catfile($repos, q[Human/NCBI36/all/bwa/someref.fa]), 'Human reference retrieved through organism field');
  is( $ruser->messages->count, 1, 'one messages');
  is( $ruser->messages->pop, 'no reference for taxon id 9606', 'no ref for taxon id message');

  my $target = catfile($repos, q[Human]);
  my $link = catfile($repos, q[taxon_ids], 9606);
  if (-e $link) { unlink $link; }
  my $symlink_exists = eval { symlink($target, $link); 1 };
  
  SKIP: {
  skip 'Cannot create taxon sym link', 2 unless $symlink_exists;
  is( pop @{$ruser->refs}, catfile($repos, q[Human/NCBI36/all/bwa/someref.fa]), 'Human reference retrieved through taxon_id');
  is( $ruser->messages->count, 0, 'no messages');
        }
  if (-e $link) { unlink $link; }

  $target = catfile($repos, q[Human], q[strain1]);
  $link = catfile($repos, q[taxon_ids], 9606);
  $symlink_exists = eval { symlink($target, $link); 1 };
  
  SKIP: {
  skip 'Cannot create taxon sym link', 2 unless $symlink_exists;
  is( pop @{$ruser->refs}, catfile($target, q[all/bwa/strain1.fa]), 'Human strain reference retrieved through taxon_id');
  is( $ruser->messages->count, 0, 'no messages');
        }
  if (-e $link) { unlink $link; }

  $target = catfile($repos, q[Human], q[strain1], q[all]);
  $link = catfile($repos, q[taxon_ids], 9606);
  $symlink_exists = eval { symlink($target, $link); 1 };
  
  SKIP: {
  skip 'Cannot create taxon sym link', 1 unless $symlink_exists;
  throws_ok {$ruser->refs} qr/wrong taxon link/, 'wrong taxon link error';
        }
  if (-e $link) { unlink $link; }
}


{
  my $ruser = npg_common::sequence::reference->new(id_run => 4254, position => 1, tag_index => 1, 
                                                   ref_repository => $repos,
                                                   search_type =>  $npg_common::sequence::reference::roles::find::SEARCH_FUZZY);
  is ((scalar @{$ruser->refs()}), 1, 'number of ref returned for a component of a multiplex with tag 1');
  is (pop @{$ruser->refs()}, catfile($repos, 'Streptococcus_pneumoniae/ATCC_700669/all/bwa/S_pneumoniae_700669.fasta'), 'ref for a component with tag 1');

  $ruser = npg_common::sequence::reference->new(id_run => 4254, position => 1, tag_index => 12, ref_repository => $repos);
  is ((scalar @{$ruser->refs()}), 0, 'no refs for tag 12 component of a multiplex since sample id in batch xml is falty');
  is ($ruser->single_ref_found, 0, 'single_ref_found returns 0');

  $ruser = npg_common::sequence::reference->new(id_run => 4254, position => 1, tag_index => 13, ref_repository => $repos);
  throws_ok {$ruser->refs()} qr/No plex description for tag 13 in run 4254, lane 1/, 'error when using a non-existing tag index';

  $ruser = npg_common::sequence::reference->new(id_run => 4254, position => 4, tag_index => 1, ref_repository => $repos);
  my $refs;
  is (scalar @{$ruser->refs()}, 1, 'single ref returned');
  is (pop @{$ruser->refs()}, catfile($repos, 'PhiX/default/all/bwa/phix-illumina.fa'), 'correct reference returned');

  $ruser = npg_common::sequence::reference->new(id_run => 4140, position => 4, tag_index => 0, ref_repository => $repos);
  is (scalar @{$ruser->refs()}, 1, 'single ref returned');
  is (pop @{$ruser->refs()}, catfile($repos, 'PhiX/default/all/bwa/phix-illumina.fa'), 'correct reference returned');
}


{
  # sample id 7575, only organism field (Human) is defined
  my $ruser = npg_common::sequence::reference->new(position => 1, id_run => 4147, ref_repository => $repos,
                            search_type =>  $npg_common::sequence::reference::roles::find::SEARCH_FUZZY);
  my @refs = @{$ruser->refs};
  is(scalar @refs, 1, 'single ref found');
  is( pop @refs, catfile($repos, q[Human/NCBI36/all/bwa/someref.fa]), 'Human reference retrieved');
  is( $ruser->messages->count, 0, 'no messages');
}


{
  my $ruser = npg_common::sequence::reference->new(id_run => 4254, position => 1, ref_repository => $repos,
                       search_type =>  $npg_common::sequence::reference::roles::find::SEARCH_FUZZY);
  is ((scalar @{$ruser->refs()}), 1, 'number of ref returned for a multiplex sample');
  is (pop @{$ruser->refs()}, catfile($repos, 'Streptococcus_pneumoniae/ATCC_700669/all/bwa/S_pneumoniae_700669.fasta'), 'single ref for a pooled lane');
}


{
  #id_run 4254
  my $batch_id = 5347;
  my $position = 1;
  my $lane;
  foreach my $l (@{st::api::batch->new({id => $batch_id,})->lanes}) {
    if ($l->position == $position) {
      $lane = $l;
      last;
    }
  }

  my $finder = Moose::Meta::Class->create_anon_class(
          roles => [qw/npg_common::sequence::reference::roles::find/])
          ->new_object({ref_repository => $repos, search_type =>  $npg_common::sequence::reference::roles::find::SEARCH_FUZZY});

  my @assets = ();
  foreach my $tag_index (keys %{$lane->pool_description}) {
    if ($lane->pool_description->{$tag_index}->{sample_id}) {
      push @assets, st::api::sample->new({id => $lane->pool_description->{$tag_index}->{sample_id},});
    } else {
      if ($lane->pool_description->{$tag_index}->{lib_tube_asset_id}) {
        push @assets, st::api::asset->new({id => $lane->pool_description->{$tag_index}->{lib_tube_asset_id},});
      }
    }
  }

  my $map = {};
  $finder->refs($map, \@assets);

  my $ref_path = catfile($repos, 'Streptococcus_pneumoniae/ATCC_700669/all/bwa/S_pneumoniae_700669.fasta');
 
  foreach my $tag_index (keys %{$lane->pool_description}) {
    if ($tag_index != 12) {
      is ($map->{$lane->pool_description->{$tag_index}->{sample_id}}, $ref_path, qq[reference for tag $tag_index]);
    }
  }
}

{
  # sample id 7575, only organism field (Human) is defined
  my $ruser = npg_common::sequence::reference->new(position => 1, id_run => 4147, ref_repository => $repos,
                    search_type =>  $npg_common::sequence::reference::roles::find::SEARCH_FUZZY);
  my @refs = @{$ruser->refs};
  is(scalar @refs, 1, 'single ref found');
  is( pop @refs, catfile($repos, q[Human/NCBI36/all/bwa/someref.fa]), 'Human reference retrieved');
  is( $ruser->messages->count, 0, 'no messages');
}

{
  my $ruser = npg_common::sequence::reference->new(id_run => 5175, position => 1, tag_index => 1, ref_repository => $repos, aligner => 'fasta');
  my $refpath =  catfile($repos, 'Bordetella_bronchiseptica/RB50/all/fasta/ref.fa');

  is (pop @{$ruser->refs()}, $refpath, 'ref_genome ref');

  my $ref_info =  $ruser->ref_info;
  isa_ok($ref_info, 'npg_common::sequence::reference::info');
  is ($ref_info->ref_path, $refpath, 'refpath field');
  is ($ref_info->aligner, q[fasta], 'aligner field');
  is ($ref_info->aligner_options, q[npg_default], 'aligner options not available');
}

{
  my $ruser = npg_common::sequence::reference->new(id_run => 4254, position => 1, ref_repository => $repos,
                   search_type =>  $npg_common::sequence::reference::roles::find::SEARCH_FUZZY);
  my $refpath = catfile($repos, 'Streptococcus_pneumoniae/ATCC_700669/all/bwa/S_pneumoniae_700669.fasta');

  is ($ruser->single_ref_found, 1, 'single_ref_found returns 1');
  is ((scalar @{$ruser->refs()}), 1, 'number of ref returned for a multiplex sample');
  is (pop @{$ruser->refs()}, $refpath, 'single ref for a pooled lane');

  my $info = npg_common::sequence::reference::info->new(ref_path => $refpath, aligner => q[bwa], aligner_options => q[-q 15]);  cmp_deeply($ruser->ref_info, $info, 'ref info object with aligner options');
}

{
  my $ruser = npg_common::sequence::reference->new(position => 1, id_run => 4147, ref_repository => $repos, search_type=>$npg_common::sequence::reference::roles::find::SEARCH_STRICT);
  is ($ruser->ref_info, undef, 'no reference info object with strict search');
  is (scalar @{$ruser->refs()}, 0, 'no references with strict search');

  $ruser = npg_common::sequence::reference->new(position => 1, id_run => 4147, ref_repository => $repos, search_type=>$npg_common::sequence::reference::roles::find::SEARCH_STRICT_AND_TAXON_ID);
  is (scalar @{$ruser->refs()}, 0, 'no references with strict-and-taxon search');
}

{
  my $ruser = npg_common::sequence::reference->new(id_run => 5175, position => 1, tag_index => 1, ref_repository => $repos, aligner => 'bwa');
  my $refpath =  catfile($repos, 'Bordetella_bronchiseptica/RB50/all/bwa/ref.fa');

  is (pop @{$ruser->refs()}, $refpath, 'ref_genome ref');

  my $ref_info =  $ruser->ref_info;
  isa_ok($ref_info, 'npg_common::sequence::reference::info');
  is ($ref_info->ref_path, $refpath, 'refpath field');
  is ($ref_info->aligner, q[bwa], 'aligner field');
  is ($ref_info->aligner_options, q{}, 'empty aligner options, use bwa default');
}

{
  my $no_align = $npg_common::sequence::reference::roles::list::NO_ALIGNMENT_OPTION;
  my $ruser = npg_common::sequence::reference->new(
                         ref_repository => $repos,
                         reference_genome => $no_align,
                         id_run => 4140, position => 4,
                         search_type => $npg_common::sequence::reference::roles::find::SEARCH_FUZZY,
                      );
  is(scalar @{$ruser->refs}, 0, 'no reference found for no-align option');
  is($ruser->search_type, $npg_common::sequence::reference::roles::find::SEARCH_STRICT, 'search mode reset correctly');
  is($ruser->messages->count, 1, 'one message saved');
  is($ruser->messages->pop, $no_align, 'correct message saved');
}

{

  my $phix = join q[/], $repos, q[PhiX/default/all/bwa/phix-illumina.fa];

  my $ruser = npg_common::sequence::reference->new(
                         ref_repository => $repos,
                         id_run => 4254,
                         position => 8,
                      );
  is( @{$ruser->refs}, 0, 'no refs for a pool lane');

  $ruser = npg_common::sequence::reference->new(
                         ref_repository => $repos,
                         id_run => 4254,
                         position => 8,
                         for_spike => 1
                      );
  is($ruser->refs->[0], $phix, 'phix ref for a pool lane with for_spike flag set');

  $ruser = npg_common::sequence::reference->new(
                         ref_repository => $repos,
                         id_run => 4254,
                         position => 8,
                         tag_index => 1,
                      );
  is( @{$ruser->refs}, 0, 'no refs for tag 1');

  $ruser = npg_common::sequence::reference->new(
                         ref_repository => $repos,
                         id_run => 4254,
                         position => 8,
                         tag_index => 168,
                      );
  is($ruser->refs->[0], $phix, 'phix ref for a tag for a spike');

  $ruser = npg_common::sequence::reference->new(
                         ref_repository => $repos,
                         id_run => 4140,
                         position => 8,
                      );
  is( @{$ruser->refs}, 0, 'no refs for a lane');

  $ruser = npg_common::sequence::reference->new(
                         ref_repository => $repos,
                         id_run => 4140,
                         position => 8,
                         for_spike => 1
                      );
  is($ruser->refs->[0], $phix, 'phix ref for a lane with for_spike flag set');
}