use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';#tests => 5;
use lib qw{t};

SKIP:
{
  eval {
    croak if !$ENV{MQ_TESTS};
  };
  skip (q{MQ_TESTS not set}, 1) if $EVAL_ERROR;
  
use_ok('npg_common::mq::stomp::topic');

## Durable subscriptions to a topic
{
  my $topic = npg_common::mq::stomp::topic->new();
  my $subscription;
  eval { $subscription = $topic->subscribe({
    topic => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{freddie_mercury},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak subscribing with a durable subscriber to selected topic});
  $topic->delete_connection();
}
{
  my $topic = npg_common::mq::stomp::topic->new();
  my $subscription;
  eval { $subscription = $topic->subscribe({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{brian_may},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak subscribing with a durable subscriber to selected topic});
  $topic->delete_connection();
}
{
  my $topic = npg_common::mq::stomp::topic->new();
  my $subscription;
  eval { $subscription = $topic->subscribe(); };
  is($EVAL_ERROR, q{}, q{no croak subscribing to defaults});
  $topic->delete_connection();
}
## sending a message to a topic
{
  my $topic = npg_common::mq::stomp::topic->new();
  isa_ok($topic, q{npg_common::mq::stomp::topic}, q{$topic});
  eval { $topic->send({}); };
  like($EVAL_ERROR, qr{No[ ]message[ ]text}, q{croak as no message/body text});
}

{
  my $topic = npg_common::mq::stomp::topic->new();

  my $arg_refs = {
    topic   => q{/topic/songs/by/queen},
    body => q{Bohemian Rhapsody},
    exchange => q{amq.topic},
    persistent => q{true},
  };
  eval { $topic->send($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak sending - topic used for destination});
}

{
  my $topic = npg_common::mq::stomp::topic->new();

  my $arg_refs = {
    destination => q{/topic/songs/by/queen},
    message => q{Killer Queen},
    exchange => q{amq.topic},
    persistent => q{true},
  };
  eval { $topic->send($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak sending - destination used for destination});
}

## getting messages from a topic using durable subscribers
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $message;
  eval { $message = $topic->get_message({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{brian_may},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak getting message with a durable subscriber to selected topic});
  is($message, q{Bohemian Rhapsody}, q{Bohemian Rhapsody first message given to brian_may});
}

{
  my $topic = npg_common::mq::stomp::topic->new();

  my $message;
  eval { $message = $topic->get_message({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{freddie_mercury},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak subscribing with a durable subscriber to selected topic});
  is($message, q{Bohemian Rhapsody}, q{Bohemian Rhapsody first message given to freddie_mercury});
}

{
  my $topic = npg_common::mq::stomp::topic->new();

  my $arg_refs = {
    topic   => q{/topic/songs/by/queen},
    body => q{A Kind of Magic},
    exchange => q{amq.topic},
    persistent => q{true},
  };
  eval { $topic->send($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak sending - topic used for destination});
}
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $message;
  eval { $message = $topic->get_message({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{freddie_mercury},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak subscribing with a durable subscriber to selected topic});
  is($message, q{Killer Queen}, q{Killer Queen second message given to freddie_mercury});
}
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $message;
  eval { $message = $topic->get_message({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{brian_may},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak getting message with a durable subscriber to selected topic});
  is($message, q{Killer Queen}, q{Killer Queen second message given to brian_may});
}
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $message;
  eval { $message = $topic->get_message({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{brian_may},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak getting message with a durable subscriber to selected topic});
  is($message, q{A Kind of Magic}, q{A Kind of Magic third message given to brian_may});
}
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $message;
  eval { $message = $topic->get_message({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{brian_may},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak getting message with a durable subscriber to selected topic});
  is($message, undef, q{No new message in topic to be given to brian_may});
}
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $message;
  eval { $message = $topic->get_message({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{freddie_mercury},
    ack => q{client},
    code => sub { my $message = shift; print "$message\n"; return { message => $message }; },
  }); };
  is($EVAL_ERROR, q{}, q{no croak getting message with a durable subscriber to selected topic});
  is(ref$message, q{HASH}, q{A Kind of Magic third message processed by code block and return value of block obtained});
  is($message->{message}, q{A Kind of Magic}, q{message in returned data structure ok});
}
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $arg_refs = {
    topic   => q{/topic/songs/by/queen},
    body => q{One Vision},
    exchange => q{amq.topic},
    persistent => q{true},
  };
  eval { $topic->send($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak sending - topic used for destination});
}
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $message;
  eval { $message = $topic->get_message({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{freddie_mercury},
    ack => q{client},
    code => sub { my $message = shift; croak q{Deliberately croaking, to see what happens.}; },
    retries => 2,
  }); };
  is($EVAL_ERROR, q{}, q{no croak getting message with code block which deliberately croaks as freddie_mercury});
  is($message, undef, q{no message or output obtained as code croaked});
}
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $message;
  eval { $message = $topic->get_message({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{freddie_mercury},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak getting message with freddie_mercury to selected topic});
  is($message, q{One Vision}, q{message One Vision obtained off topic});
}
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $message;
  eval { $message = $topic->get_message({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{brian_may},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak getting message with brian_may to selected topic});
  is($message, q{One Vision}, q{message One Vision obtained off topic});
}
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $arg_refs = {
    topic   => q{/topic/songs/by/queen},
    body => q{Hammer to Fall},
    exchange => q{amq.topic},
    persistent => q{true},
  };
  eval { $topic->send($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak sending - topic used for destination});
}
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $message;
  eval { $message = $topic->get_message({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{freddie_mercury},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak getting message with freddie_mercury to selected topic});
  is($message, q{Hammer to Fall}, q{message Hammer to Fall obtained off topic});
}
{
  my $topic = npg_common::mq::stomp::topic->new();

  my $message;
  eval { $message = $topic->get_message({
    destination => q{/topic/songs/by/queen},
    durable => q{true},
    client_id => q{brian_may},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak getting message with brian_may to selected topic});
  is($message, q{Hammer to Fall}, q{message Hammer to Fall obtained of topic});
}

} # end of total SKIP block
1;
