use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';#tests => 5;
use lib qw{t};
SKIP:
{
  eval {
    croak if !$ENV{MQ_TESTS};
  };
  skip (q{MQ_TESTS not set}, 1) if $EVAL_ERROR;
  
use_ok('npg_common::mq::stomp::topic');
my $maxnummsg=50;

## Durable subscriptions to a topic
{
  my $topic = npg_common::mq::stomp::topic->new();
  my $subscription;
  eval { $subscription = $topic->subscribe({
    topic => q{/topic/big.pending},
    durable => q{true},
    client_id => q{bp_client1},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{no croak subscribing with a durable subscriber to selected topic});
  $topic->delete_connection();
}
## sending a message to a topic
{
  my $topic = npg_common::mq::stomp::topic->new();
  my %eval_error;
  foreach my $i (1..$maxnummsg){
    my $arg_refs = {
      topic   => q{/topic/big.pending},
      body => q{message }.$i,
      exchange => q{amq.topic},
      persistent => q{true},
    };
    eval { $topic->send($arg_refs); };
    $eval_error{$EVAL_ERROR}++;
  }
  is($eval_error{q()}, $maxnummsg, qq{no croak sending $maxnummsg});
}


## getting messages from a topic using durable subscribers
{
  my $topic = npg_common::mq::stomp::topic->new();

  my %eval_error;
  my $mcount=0;
  my %messages;
  while(1){
    my $message;
    eval { $message = $topic->get_message({
      destination => q{/topic/big.pending},
      durable => q{true},
      client_id => q{bp_client1},
      ack => q{client},
    }); };
    $eval_error{$EVAL_ERROR}++;
    last unless $message;
    $mcount++;
    $messages{$message}++;
  }
  is($eval_error{q()}, $maxnummsg+1, qq{no croak received X ($maxnummsg +1)});
  is($mcount, $maxnummsg, qq{message received X $maxnummsg });
  is($mcount, $maxnummsg, qq{message received X $maxnummsg });
  is(scalar(grep{$messages{$_}!=1} keys %messages), 0, q{no messages received more than once});
}

} # end of total SKIP block
1;
