#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: ajb $
# Created:       16 June 2010
# Last Modified: $Date: 2011-07-21 16:26:34 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 60-request-multicache.t 13725 2011-07-21 15:26:34Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-37.0/t/60-request-multicache.t $
#


use strict;
use warnings;
use Test::More tests => 79;
use Test::Exception;
use Test::Deep;
use File::Temp qw/ tempdir /;
use File::Spec::Functions qw(catfile);
use npg_testing::intweb qw(npg_is_accessible);

use npg::api::run;
use npg::api::util;

use_ok('npg_common::request');

{
  SKIP: {

  if (!npg_is_accessible()) {
     skip 'Live test, but sanger intweb is not accessible',  78;
  }

  my $temp =  tempdir( CLEANUP => 1 );
  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = $temp;

  foreach my $save2cache_flag (qw(1 0)) {
    my $util = npg::api::util->new({save2cache => $save2cache_flag,});

    my $run = npg::api::run->new({
				     max_retries => 1,
				     retry_delay => 1,
				     util        => $util,
				     id_run      => 3905,                            
				    });
    isa_ok($run, 'npg::api::run', 'run isa');

    # Test the fields.
    cmp_ok($run->batch_id(),  q(==), 4775, 'run batch_id');
    cmp_ok($run->id_run(),    q(==), 3905, 'run id_run');
    cmp_ok($run->name(),      q(eq), 'IL19_3905', 'run name');

    #get the batch
    my $batch = $run->batch();
    isa_ok($batch, 'st::api::batch', 'batch isa');
    cmp_ok($batch->lane_count(),  q(==), 8, 'batch lane count');

    {#get the lanes from the batch
    my @lanes = @{$batch->lanes()};
    isa_ok($lanes[0], 'st::api::lane', 'first st lane isa');
    cmp_ok($lanes[0]->position, q(==), 1, 'first st lane position');
    cmp_ok($lanes[5]->position, q(==), 6, 'sixth st lane position');
    is($lanes[0]->control, undef, 'first st lane has no control');
    is($lanes[0]->pool, undef, 'first st lane has no pool');
    is($lanes[3]->library, undef, 'fourth st lane has no library');
    is($lanes[3]->pool, undef, 'fourth st lane has no pool');
    my $entity = $lanes[0]->entity;
    isa_ok($entity, 'st::api::asset', 'first st lane entity isa');
    cmp_ok($entity->id, q(==), 57440, 'entity id from first st lane');
    cmp_ok($entity->name, q(eq), 'PD3918a 1', 'entity name from first st lane');
    my $library = $lanes[0]->library;
    isa_ok($library, 'st::api::asset', 'first st lane library isa');
    cmp_ok($library->id, q(==), 57440, 'library id from first st lane');
    my $control = $lanes[3]->control;
    isa_ok($control, 'st::api::asset', 'fourth st lane control isa');
    cmp_ok($control->id, q(==), 79577, 'control id from fourth st lane');
    cmp_ok($control->reference_sequence, q(eq), '/software/solexa/src/references/PhiX/Illumina/all/fasta/phix-illumina.fa', 'control reference seq from fourth st lane');
    my $project;
    lives_ok {$project = $lanes[5]->entity->project;} 'project for sixth st lane';
    lives_ok {$project = $lanes[5]->entity->study;} 'study for sixth st lane';
    is ($lanes[0]->entity->sample->name, 'PD3918a', 'sample name');
    
    cmp_ok($project->id, q(==), 333, 'study id');
    cmp_ok($project->name, q(eq), q(CLL whole genome), 'study name');
 warn @{$project->email_addresses};
    is_deeply($project->email_addresses,[qw(dg10@sanger.ac.uk las@sanger.ac.uk pc8@sanger.ac.uk sm2@sanger.ac.uk)],'All email addresses');
    is_deeply($project->email_addresses_of_managers,[qw(sm2@sanger.ac.uk)],'Managers email addresses');
    is_deeply($project->email_addresses_of_followers,[qw(dg10@sanger.ac.uk las@sanger.ac.uk pc8@sanger.ac.uk)],'Followers email addresses');
    is_deeply($project->email_addresses_of_owners,[qw(sm2@sanger.ac.uk)],'Owners email addresses');
    }

    #get npg run lanes from run
    my @run_lanes = @{$run->run_lanes};
    cmp_ok(@run_lanes, q(==), 8, 'number of run lanes');
    isa_ok($run_lanes[0], 'npg::api::run_lane', 'first run lane isa');
    cmp_ok($run_lanes[0]->position, q(==), 1, 'first run lane position');
    cmp_ok($run_lanes[5]->position, q(==), 6, 'sixth run lane position');
    is($run_lanes[0]->control, undef, 'first run lane is not control asset');
    my $control = $run_lanes[3]->control;
    isa_ok($control, 'st::api::asset', 'fourth run lane is a control asset');

    if ($save2cache_flag == 1) {
      my $supports_cache = 0;
      eval {
        $util->save2cache(1);
        $supports_cache = 1;
      };
      SKIP: {
        if (!$supports_cache) {
          skip 'Cache check skipped, npg and st apis do not support caching', 6;
        }

        my @files = qw( npg/run/3905.xml
                      st/batches/4775.xml
                      st/assets/79577.xml
                      st/samples/7283.xml
                      st/samples/9836.xml
                      st/studies/333.xml
                      );

        foreach my $file (@files) {
          ok(-e catfile($ENV{NPG_WEBSERVICE_CACHE_DIR}, $file), qq[$file exists in cache]);
        }
      }
    }

  } #end of foreach loop
  } #end of SKIP
}